
# bedrock-rnd - curated footprints, symbols and devmaps for Ringdove EDA
# Copyright (C) 2025  Tibor 'Igor2' Palinkas
#
# dist-license: GPL2+
# use-license (license of the output): unlimited (CC0 or public domain)
#
# For more information: doc/license.html
#        or on the web: http://www.repo.hu/projects/bedrock-rnd/license.html
#
# Project page: http://www.repo.hu/projects/bedrock-rnd
#

# Draw gullwing and J packages

# fill in PINPAT[]; it is addressed as if there were all 6 or 8 pins available
# and it is '1' for the virtual pins that exist in reality. For example
# the classic sot-23-3 is stoed as a 6-pin device with pins 1, 3 and 5
# exist
function sot23_pinpat(PINPAT, pinno,     n)
{
	pinno = int(pinno)
	PINPAT["rangen"] = -1
	PINPAT["rangep"] = +1
	PINPAT[1] = 1;
	PINPAT[3] = 1;
	if (pinno == 3) {
		PINPAT[5] = 1;
	}
	else if (pinno == 4) {
		PINPAT[4] = 1;
		PINPAT[6] = 1;
	}
	else if (pinno == 5) {
		PINPAT[2] = 1;
		PINPAT[4] = 1;
		PINPAT[6] = 1;
	}
	else if (pinno == 6) {
		PINPAT[2] = 1;
		PINPAT[4] = 1;
		PINPAT[5] = 1;
		PINPAT[6] = 1;
	}
	else if (pinno == 8) {
		PINPAT["rangen"] = -1.5
		PINPAT["rangep"] = +1.5
		for(n = 1; n <= 8; n++)
			PINPAT[n] = 1;
	}
}

function so_pinpat(PINPAT, pinno,     n)
{
	pinno = int(pinno)
	if ((pinno % 2) != 0) {
		print "Invalid N (pin number) for SO part (gull-wing type with pins on 2 sides): number of pins must be even" > "/dev/stderr"
		exit 1
	}
	PINPAT["rangen"] = -pinno/4+0.5
	PINPAT["rangep"] = +pinno/4-0.5
	for(n = 1; n <= pinno; n++)
		PINPAT[n] = 1;
}

# figure number of pins for square or rectangular: -ipc (body specified) case
function square_vs_rectangle_body()
{
	if ((P["NL"] == "") || (P["NW"] == "")) {
		# square: N specified
		if ((P["NL"] != "") || (P["NW"] != "")) {
			print "Either specify both NL and NW or don't specify either of them" > "/dev/stderr"
			exit 1
		}
		if (P["N"] == "") {
			print "number if pins unknown: specify either N for total number of pins (when NL and NW are not specified)" > "/dev/stderr"
			exit 1
		}
		if ((P["PE"] == "") || (P["PEL"] != "") || (P["PEW"] != "")) {
			print "square body package needs PE specified (and no PEL or PEW specified)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["PE"],  "PE")
		copy_dim("PEL", "PE")
		copy_dim("PEW", "PE")

		if ((P["BS"] == "") || (P["BL"] != "") || (P["BW"] != "")) {
			print "square body package needs BS specified (and no BL or BW specified)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["BS"],  "BS")
		copy_dim("BL", "BS")
		copy_dim("BW", "BS")

		P["N"] = int(P["N"])
		if ((P["N"] % 4) != 0) {
			print "N needs to be a divisible by 4 for square package" > "/dev/stderr"
			exit 1
		}
		P["NL"] = P["N"]/4
		P["NW"] = P["N"]/4
	}
	else {
		# rectangular: NL and NW specified
		if (P["N"] != "") {
			print "do not specify N when NL and NW are specified" > "/dev/stderr"
			exit 1
		}
		if ((P["PEL"] == "") || (P["PEW"] == "")) {
			print "PEL and PEW must be specified (instead of PE) for rectangular package" > "/dev/stderr"
			exit 1
		}

		if ((P["PE"] != "") || (P["PEL"] == "") || (P["PEW"] == "")) {
			print "rectangular body package needs PEL and PEW specified (and no PE)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["PEL"],  "PEL")
		load_dim(P["PEW"],  "PEW")

		if ((P["BS"] != "") || (P["BL"] == "") || (P["BW"] == "")) {
			print "rectangular body package needs BL and BW specified (and no BS)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["BL"],  "BL")
		load_dim(P["BW"],  "BW")

		P["NL"] = int(P["NL"])
		P["NW"] = int(P["NW"])
	}
}


# figure number of pins for square or rectangular: -rfp (pad specified) case
function square_vs_rectangle_pads()
{
	if ((P["NL"] == "") || (P["NW"] == "")) {
		# square: N specified
		if ((P["NL"] != "") || (P["NW"] != "")) {
			print "Either specify both NL and NW or don't specify either of them" > "/dev/stderr"
			exit 1
		}
		if (P["N"] == "") {
			print "number if pins unknown: specify either N for total number of pins (when NL and NW are not specified)" > "/dev/stderr"
			exit 1
		}
		if ((P["PRP"] == "") || (P["PRPL"] != "") || (P["PRPW"] != "")) {
			print "square body package needs PRP specified (and no PRPL or PRPW specified)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["PRP"],  "PRP")
		copy_dim("PRPL", "PRP")
		copy_dim("PRPW", "PRP")

		P["N"] = int(P["N"])
		if ((P["N"] % 4) != 0) {
			print "N needs to be a divisible by 4 for square package" > "/dev/stderr"
			exit 1
		}
		P["NL"] = P["N"]/4
		P["NW"] = P["N"]/4

		load_dim(P["BS"],  "BS")
		copy_dim("BL", "BS");
		copy_dim("BW", "BS");


		load_dim(P["PE"],  "PE")
		if (P["PEL"] == "") {
			P["PEL"] = P["PE"]
			copy_dim("PEL", "PE");
		}
		if (P["PEW"] == "") {
			P["PEW"] = P["PE"]
			copy_dim("PEW", "PE");
		}
	}
	else {
		# rectangular: NL and NW specified
		if (P["N"] != "") {
			print "do not specify N when NL and NW are specified" > "/dev/stderr"
			exit 1
		}
		if ((P["PRPL"] == "") || (P["PRPW"] == "")) {
			print "PRPL and PRPW must be specified (instead of PRP) for rectangular package" > "/dev/stderr"
			exit 1
		}

		if ((P["PRP"] != "") || (P["PRPL"] == "") || (P["PRPW"] == "")) {
			print "rectangular body package needs PRPL and PRPW specified (and no PRP)" > "/dev/stderr"
			exit 1
		}
		load_dim(P["PRPL"],  "PRPL")
		load_dim(P["PRPW"],  "PRPW")

		P["NL"] = int(P["NL"])
		P["NW"] = int(P["NW"])

		load_dim(P["BL"],  "BL")
		load_dim(P["BW"],  "BW")
	}
}

### internal utility ###

function asm_gull_wing(x_inner, y_center, x_outer, width, x_bend    ,r,y1,y2,xb)
{
	r = width/2
	y1 = y_center-r
	y2 = y_center+r


	asm_line(x_outer, y1, x_outer, y2, 0.05)
	asm_line(x_outer, y1, x_inner, y1, 0.05)
	asm_line(x_outer, y2, x_inner, y2, 0.05)

	if (x_bend == "FN")
		return

	if (x_bend == "J") {
		asm_line(x_inner, y1, x_inner, y2, 0.05)
	}
	else if (x_bend != "") {
		r = r * 0.7
		y1 = y_center-r
		y2 = y_center+r

		asm_line(x_bend, y1, x_bend, y2, 0.025)
	}
}

function asm_gull_wing90(y_inner, x_center, y_outer, width, y_bend    ,r,x1,x2,yb)
{
	r = width/2
	x1 = x_center-r
	x2 = x_center+r


	asm_line(x1, y_outer, x2, y_outer, 0.05)
	asm_line(x1, y_outer, x1, y_inner, 0.05)
	asm_line(x2, y_outer, x2, y_inner, 0.05)

	if (y_bend == "FN")
		return

	if (y_bend == "J") {
		asm_line(x1, y_inner, x2, y_inner, 0.05)
	}
	else if (y_bend != "") {
		r = r * 0.7
		x1 = x_center-r
		x2 = x_center+r

		asm_line(x1, y_bend, x2, y_bend, 0.025)
	}
}

### main entry point: draw gullwing footprints ###

# Draw a gull-wing and J part with pins on two sides
function gullwing2_draw(D, PINPAT, pinshape      ,y,yy,id,left_y1,left_y2,right_y1,right_y2,x,cl,cye,ppid,pad_proto,cpad_proto,cid)
{

	ipc_subc_begin("", "Q?", -D["body_width_max"]/2, -D["body_length_max"]/2 - mm(2), 0, SUBC_ATTR)
	ipc_subc_layers()

# create the padstack prototype of the only pad we need
	pad_proto = ipc_subc_proto_begin()
	subc_pstk_add_shape_rsquare(pad_proto, "top-copper", 0, 0, mm(D["pad_length"]), mm(D["pad_width"]), pad_rounding)
	subc_pstk_add_shape_rsquare(pad_proto, "top-mask", 0, 0, mm(D["pad_length"])+mask_offs*2, mm(D["pad_width"])+mask_offs*2, inc_radius(pad_rounding, mask_offs))
	subc_pstk_add_shape_rsquare(pad_proto, "top-paste", -0.025, 0, mm(D["pad_length"])-0.1, mm(D["pad_width"])-0.05, inc_radius(pad_rounding, -0.05))
	ipc_subc_proto_end(pad_proto)

	centerpad(D)

# draw pins
	for(y = PINPAT["rangen"]; y <= PINPAT["rangep"]; y++) {
		if (PINPAT[++ppid]) {
			id++
			yy = y*D["pitch"]
			subc_pstk(pad_proto, mm(-D["pad_spacing"]/2), mm(yy), 0, id, id, 0.2)
			if (left_y1 == "") left_y1 = yy
			left_y2 = yy
			if (pinshape == "FN")
				asm_gull_wing(-D["body_width"]/2, yy, -D["lead_gap"]/2, D["lead_width"], "FN")
			else if (pinshape == "J")
				asm_gull_wing(-D["lead_gap"]/2, yy, -D["lead_extent"]/2, D["lead_width"], "J")
			else
				asm_gull_wing(-D["body_width"]/2, yy, -D["lead_extent"]/2, D["lead_width"], -D["lead_gap"]/2)
		}
	}

# pin 1 mark on assy
	if (pinshape == "J")
		asm_line(-D["lead_gap"]/2 + 0.5, left_y1, -D["lead_gap"]/2 + 0.5, left_y1, 0.25)
	else
		asm_line(-D["body_width"]/2+0.35, -D["body_length"]/2+0.35, -D["body_width"]/2+0.35, -D["body_length"]/2+0.35, 0.25)

	for(y = PINPAT["rangep"]; y >= PINPAT["rangen"]; y--) {
		if (PINPAT[++ppid]) {
			id++
			yy = y*D["pitch"]
			subc_pstk(pad_proto, mm(+D["pad_spacing"]/2), mm(yy), 180, id, id, 0.2)
			if (right_y1 == "") right_y1 = yy
			right_y2 = yy
			if (pinshape == "FN")
				asm_gull_wing(+D["body_width"]/2, yy, +D["lead_gap"]/2, D["lead_width"], "FN")
			else if (pinshape == "J")
				asm_gull_wing(+D["lead_gap"]/2, yy, +D["lead_extent"]/2, D["lead_width"], "J")
			else
				asm_gull_wing(+D["body_width"]/2, yy, +D["lead_extent"]/2, D["lead_width"], +D["lead_gap"]/2)
		}
	}

# draw courtyard
	cye = D["courtyard_extent"]

# body courtyard
	courtyard_add_rect(-D["body_width_max"]/2 - cye, -D["body_length_max"]/2 - cye, +D["body_width_max"]/2 + cye, +D["body_length_max"]/2 + cye)

# pad row courtyars
	courtyard_add_rect(-D["pad_spacing"]/2-D["pad_length"]/2-cye, left_y1-D["pad_width"]/2-cye, -D["pad_spacing"]/2+D["pad_length"]/2+cye, left_y2+D["pad_width"]/2+cye)
	courtyard_add_rect(+D["pad_spacing"]/2-D["pad_length"]/2-cye, right_y2-D["pad_width"]/2-cye, +D["pad_spacing"]/2+D["pad_length"]/2+cye, right_y1+D["pad_width"]/2+cye)

	courtyard_print()

# draw body
	asm_body_box(-D["body_width"]/2, -D["body_length"]/2, +D["body_width"]/2, +D["body_length"]/2)


# draw adhesive
	adhesive_line(0, -D["body_length"]*0.25, 0, +D["body_length"]*0.25)

# draw silk graphics
	x = D["body_width_max"]/2 + silk_thick/2
	y = D["body_length_max"]/2 + silk_thick/2

	silk_line(-x, y, +x, y)
	silk_line(-x, -y, +x, -y)

	if (P["N"] == 3) {
		cl = D["body_length_max"]/2-D["lead_width_max"]-silk_thick
		silk_line(x, -y, x, -y+cl)
		silk_line(x, +y, x, +y-cl)
	}

	silk_pin1(-x - 0.5, -y)

	subc_end()
}

# Draw a gull-wing part with pins on four sides
function gullwing4_draw(D, pinshape,      y,yy,x,xx,id,cmin,cmax,cl,cye,ppid,pad_proto,cpad_proto,cid,silk_x,silk_y,idmax)
{
	ipc_subc_begin("", "Q?", +D["body_width_max"]/2, -D["body_length_max"]/2-mm(2), 0, SUBC_ATTR)
	ipc_subc_layers()

# create the padstack prototype of the only pad we need
	pad_proto = ipc_subc_proto_begin()
	subc_pstk_add_shape_rsquare(pad_proto, "top-copper", 0, 0, mm(D["pad_length"]), mm(D["pad_width"]), pad_rounding)
	subc_pstk_add_shape_rsquare(pad_proto, "top-mask", 0, 0, mm(D["pad_length"])+mask_offs*2, mm(D["pad_width"])+mask_offs*2, inc_radius(pad_rounding, mask_offs))
	subc_pstk_add_shape_rsquare(pad_proto, "top-paste", -0.025, 0, mm(D["pad_length"])-0.1, mm(D["pad_width"])-0.05, inc_radius(pad_rounding, -0.05))
	ipc_subc_proto_end(pad_proto)

	centerpad(D)

	cye = D["courtyard_extent"]
	silk_x = D["body_width_max"]/2 + silk_thick/2
	silk_y = D["body_length_max"]/2 + silk_thick/2

	if (D["pin1_top"]) {
		id = int(P["NW"]/2+0.5)
		idmax = P["NL"]*2 + P["NW"]*2
	}

	silk_backoff = D["pad_width"]*0.5+mask_offs+silk_thick/2
#	print "silk_backoff =", silk_backoff, D["pad_width"]*0.5-mask_offs-silk_thick/2 > "/dev/stderr"

# draw pins on west side
	cmin=""
	for(y = -(P["NL"]-1)/2; y <= +(P["NL"]-1)/2; y++) {
		id++
		yy = y*D["pitch"]
		subc_pstk(pad_proto, mm(-D["pad_spacing_width"]/2), mm(yy), 0, id, id, 0.2)
		if (cmin == "") cmin = yy
		cmax = yy
		if (pinshape == "FN")
			asm_gull_wing(-D["body_width"]/2, yy, -D["lead_gap_width"]/2, D["lead_width"], "FN")
		else if (pinshape == "J")
			asm_gull_wing(-D["lead_gap_width"]/2, yy, -D["lead_extent_width"]/2, D["lead_width"], "J")
		else
			asm_gull_wing(-D["body_width"]/2, yy, -D["lead_extent_width"]/2, D["lead_width"], -D["lead_gap_width"]/2)
	}
	courtyard_add_rect(-D["pad_spacing_width"]/2-D["pad_length"]/2-cye, cmin-D["pad_width"]/2-cye, -D["pad_spacing_width"]/2+D["pad_length"]/2+cye, cmax+D["pad_width"]/2+cye)
	silk_line(-silk_x, -silk_y, -silk_x, cmin-silk_backoff)
	silk_line(-silk_x, +silk_y, -silk_x, cmax+silk_backoff)

# pin 1 mark on silk and assy
	if (pinshape == "FN") {
		silk_pin1(-D["body_width"]/2-0.4, cmin-D["pad_width"]*1.1-0.1)
		asm_line(-D["lead_gap_width"]/2+0.125, cmin, -D["lead_gap_width"]/2+0.125, cmin, 0.25)
	}
	else if (D["pin1_top"]) {
		silk_pin1(0, -D["lead_extent_length"]/2-1)
		asm_line(0, -D["lead_gap_length"]/2 + 1, 0, -D["lead_gap_length"]/2 + 1, 0.25)
	}
	else {
		silk_pin1(-D["pad_spacing_width"]/2, cmin-D["pad_width"]*1.1)
		asm_line(-D["body_width"]/2+0.35, cmin, -D["body_width"]/2+0.35, cmin, 0.25)
	}


# draw pins on south side
	cmin=""
	for(x = -(P["NW"]-1)/2; x <= +(P["NW"]-1)/2; x++) {
		id++
		xx = x*D["pitch"]
		subc_pstk(pad_proto, mm(xx), mm(+D["pad_spacing_length"]/2), 90, id, id, 0.2)
		if (cmin == "") cmin = xx
		cmax = xx
		if (pinshape == "FN")
			asm_gull_wing90(+D["body_length"]/2, xx, +D["lead_gap_length"]/2, D["lead_width"], "FN")
		else if (pinshape == "J")
			asm_gull_wing90(+D["lead_gap_length"]/2, xx, +D["lead_extent_length"]/2, D["lead_width"], "J")
		else
			asm_gull_wing90(+D["body_length"]/2, xx, +D["lead_extent_length"]/2, D["lead_width"], +D["lead_gap_length"]/2)
	}
	courtyard_add_rect(cmin-D["pad_width"]/2-cye, +D["pad_spacing_length"]/2-D["pad_length"]/2-cye, cmax+D["pad_width"]/2+cye, +D["pad_spacing_length"]/2+D["pad_length"]/2+cye)
	silk_line(-silk_x, +silk_y, cmin-silk_backoff, +silk_y)
	silk_line(+silk_x, +silk_y, cmax+silk_backoff, +silk_y)


# draw pins on east side
	cmin=""
	for(y = +(P["NL"]-1)/2; y >= -(P["NL"]-1)/2; y--) {
		id++
		yy = y*D["pitch"]
		subc_pstk(pad_proto, mm(+D["pad_spacing_width"]/2), mm(yy), 180, id, id, 0.2)
		if (cmin == "") cmin = yy
		cmax = yy
		if (pinshape == "FN")
			asm_gull_wing(+D["body_width"]/2, yy, +D["lead_gap_width"]/2, D["lead_width"], "FN")
		else if (pinshape == "J")
			asm_gull_wing(+D["lead_gap_width"]/2, yy, +D["lead_extent_width"]/2, D["lead_width"], "J")
		else
			asm_gull_wing(+D["body_width"]/2, yy, +D["lead_extent_width"]/2, D["lead_width"], +D["lead_gap_width"]/2)
	}
	courtyard_add_rect(+D["pad_spacing_width"]/2-D["pad_length"]/2-cye, cmax-D["pad_width"]/2-cye, +D["pad_spacing_width"]/2+D["pad_length"]/2+cye, cmin+D["pad_width"]/2+cye)
	silk_line(+silk_x, -silk_y, +silk_x, cmax-silk_backoff)
	silk_line(+silk_x, +silk_y, +silk_x, cmin+silk_backoff)

# draw pins on north side
	cmin=""
	for(x = +(P["NW"]-1)/2; x >= -(P["NW"]-1)/2; x--) {
		id++
		if (D["pin1_top"] && (id > idmax))
			id = 1
		xx = x*D["pitch"]
		subc_pstk(pad_proto, mm(xx), mm(-D["pad_spacing_length"]/2), 270, id, id, 0.2)
		if (cmin == "") cmin = xx
		cmax = xx
		if (pinshape == "FN")
			asm_gull_wing90(-D["body_length"]/2, xx, -D["lead_gap_length"]/2, D["lead_width"], "FN")
		else if (pinshape == "J")
			asm_gull_wing90(-D["lead_gap_length"]/2, xx, -D["lead_extent_length"]/2, D["lead_width"], "J")
		else
			asm_gull_wing90(-D["body_length"]/2, xx, -D["lead_extent_length"]/2, D["lead_width"], -D["lead_gap_length"]/2)
	}
	courtyard_add_rect(cmax-D["pad_width"]/2-cye, -D["pad_spacing_length"]/2-D["pad_length"]/2-cye, cmin+D["pad_width"]/2+cye, -D["pad_spacing_length"]/2+D["pad_length"]/2+cye)
	silk_line(-silk_x, -silk_y, cmax-silk_backoff, -silk_y)
	silk_line(+silk_x, -silk_y, cmin+silk_backoff, -silk_y)

	if (D["pin1_top"]) {
		silk_line(-silk_x, -silk_y+1, cmax-silk_backoff, -silk_y+1)
		silk_line(+silk_x, -silk_y+1, cmin+silk_backoff, -silk_y+1)
	}


# body courtyard
	courtyard_add_rect(-D["body_width_max"]/2 - cye, -D["body_length_max"]/2 - cye, +D["body_width_max"]/2 + cye, +D["body_length_max"]/2 + cye)
	courtyard_print()

# draw body
	if (D["pin1_top"])
		asm_body_box(-D["body_width"]/2, -D["body_length"]/2, +D["body_width"]/2, +D["body_length"]/2, 0.1, 1.5)
	else
		asm_body_box(-D["body_width"]/2, -D["body_length"]/2, +D["body_width"]/2, +D["body_length"]/2)


# draw silk graphics
#	silk_line(-silk_x, +silk_y, +silk_x, +silk_y)
#	silk_line(-silk_x, -silk_y, +silk_x, -silk_y)


	subc_end()
}
