
# bedrock-rnd - curated footprints, symbols and devmaps for Ringdove EDA
# Copyright (C) 2025  Tibor 'Igor2' Palinkas
#
# dist-license: GPL2+
# use-license (license of the output): unlimited (CC0 or public domain)
#
# For more information: doc/license.html
#        or on the web: http://www.repo.hu/projects/bedrock-rnd/license.html
#
# Project page: http://www.repo.hu/projects/bedrock-rnd
#

function subc_pstk_add_shape_rsquare_reduced(proto, layer, x, y, sx, sy, r, red_factor    ,s,minc,steps,astep)
{
	if (r+0 <= 0)
		r = 0.05

# clamp at minimum to avoid broken polygons (can't do squares smaller than 4 nanometer)
	minc = mm(0.000004)
	if (sx < minc) sx = minc;
	if (sy < minc) sy = minc;

# how many steps an 1/4th circle should have so that line length is 0.5mm
	if (r > 5)
		steps = 8
	else if (r >= 1)
		steps = 6
	else if (r >= 0.5)
		steps = 4
	else if (r >= 0.1)
		steps = 3
	else if (r >= 0.02)
		steps = 2
	else
		steps = 0

# angle stepping
	if (steps > 0)
		astep = (pi/2) / steps

	sx = sx / 2
	sy = sy / 2
	s = s "      ha:ps_shape_v4 {" NL
	s = s "       clearance = 0" NL
	s = s "       li:ps_poly {" NL

	# north side
	s = s "        " unit(x - sx+r) ";" unit(y - sy) ";" NL
	s = s "        " unit(x + sx-r) ";" unit(y - sy) ";" NL

	# north-east corner
	if (steps > 0)
		s = s "        " poly_arc(x + sx-r, y - sy+r, r, 1.5*pi, 2*pi, astep) NL

	# east
	s = s "        " unit(x + sx) ";" unit(y - sy+r) ";" NL
	s = s "        " unit(x + sx - sx*red_factor) ";" unit(y) ";" NL
	s = s "        " unit(x + sx) ";" unit(y + sy-r) ";" NL

	# south-east corner
	if (steps > 0)
		s = s "        " poly_arc(x + sx-r, y + sy-r, r, 0, 0.5*pi, astep) NL


	# south
	s = s "        " unit(x + sx-r) ";" unit(y + sy) ";" NL
	s = s "        " unit(x - sx+r) ";" unit(y + sy) ";" NL

	# south-west corner
	if (steps > 0)
		s = s "        " poly_arc(x - sx+r, y + sy-r, r, 0.5*pi, 1.0*pi, astep) NL

	# sest
	s = s "        " unit(x - sx) ";" unit(y + sy-r) ";" NL
	s = s "        " unit(x - sx) ";" unit(y - sy+r) ";" NL

	# north-west corner
	if (steps > 0)
		s = s "        " poly_arc(x - sx+r, y - sy+r, r, 1.0*pi, 1.5*pi, astep) NL


	s = s "       }" NL
	s = s subc_pstk_shape_layer(layer)
	s = s "      }" NL
	PROTO[proto] = PROTO[proto] s
}


# Draw a rectangular pad passive
function rp_pas_draw(D,     pad_proto,x,y)
{

	ipc_subc_begin("", "U?", 0, -D["lead_width"]/2 - mm(2), 0, SUBC_ATTR)
	ipc_subc_layers()

# create the padstack prototype of the only pad we need
	pad_proto = ipc_subc_proto_begin()
	subc_pstk_add_shape_rsquare(pad_proto, "top-copper", 0, 0, mm(D["pad_length"]), mm(D["pad_width"]), pad_rounding)
	subc_pstk_add_shape_rsquare(pad_proto, "top-mask", 0, 0, mm(D["pad_length"])+mask_offs*2, mm(D["pad_width"])+mask_offs*2, inc_radius(pad_rounding, mask_offs))
	if (PADGEO["paste_reduction"])
		subc_pstk_add_shape_rsquare_reduced(pad_proto, "top-paste", -0.025, 0, mm(D["pad_length"])-0.1, mm(D["pad_width"])-0.05, inc_radius(pad_rounding, -0.05), 0.175)
	else
		subc_pstk_add_shape_rsquare(pad_proto, "top-paste", 0, 0, mm(D["pad_length"])-0.05, mm(D["pad_width"])-0.05, inc_radius(pad_rounding, -0.05))
	ipc_subc_proto_end(pad_proto)

	subc_pstk(pad_proto, -D["pad_spacing"]/2, 0, 0,     1, 1,   0.2)
	subc_pstk(pad_proto, +D["pad_spacing"]/2, 0, 180,   2, 2,   0.2)

# draw courtyard
	cye = D["courtyard_extent"]
	courtyard_add_rect(-D["pad_spacing"]/2 - D["pad_length"]/2 - cye, -D["pad_width"]/2 - cye, +D["pad_spacing"]/2 + D["pad_length"]/2 + cye, +D["pad_width"]/2 + cye)
	courtyard_print()

# draw body
	asm_box(-D["lead_extent"]/2, -D["lead_width"]/2, +D["lead_extent"]/2, +D["lead_width"]/2)
	asm_line(-D["lead_gap"]/2, -D["lead_width"]/2, -D["lead_gap"]/2, +D["lead_width"]/2)
	asm_line(+D["lead_gap"]/2, -D["lead_width"]/2, +D["lead_gap"]/2, +D["lead_width"]/2)

# draw adhesive
	adhesive_line(0, -D["lead_width"]*0.25, 0, +D["lead_width"]*0.25)

# draw silk graphics
	x = D["pad_spacing"]/2 - D["pad_length"]/2 - silk_thick*0.75
	y = D["lead_width"]/2 + silk_thick*0.5

	if (x > silk_thick) {
		silk_line(-x, y, +x, y)
		silk_line(-x, -y, +x, -y)
	}

# TODO: polarity mark
#	silk_pin1(-x - 0.5, -y)
#		asm_line(-D["body_width"]/2+0.35, -D["body_length"]/2+0.35, -D["body_width"]/2+0.35, -D["body_length"]/2+0.35, 0.25)

	subc_end()
}

