
# bedrock-rnd - curated footprints, symbols and devmaps for Ringdove EDA
# Copyright (C) 2025  Tibor 'Igor2' Palinkas
#
# dist-license: GPL2+
# use-license (license of the output): unlimited (CC0 or public domain)
#
# For more information: doc/license.html
#        or on the web: http://www.repo.hu/projects/bedrock-rnd/license.html
#
# Project page: http://www.repo.hu/projects/bedrock-rnd
#

BEGIN {
	gversion="2025-01-20"
	generator = "dfn-ipc"
	bedrock_ver = "2025-01-08a"
}

BEGIN {
	help_auto()

	proc_args(P, "density,N,BL,BW,PP,PE,PT,PC,PG,ECPL,ECPW" ipc_params, "density,N,PP,PT")


# pin extent is the same as body size
	if ("BS" in P) P["PE"] = P["BS"]
	if ("BW" in P) P["PE"] = P["BW"]
	P["PRP"] = P["PE"]

# load args
	load_dim(P["BL"],  "BL")
	load_dim(P["BW"],  "BW")
	load_dim(P["PP"],  "PP")
	load_dim(P["PE"],  "PE")
	load_dim(P["PT"],  "PT")
	load_dim(P["PC"],  "PC")
	load_dim(P["PG"],  "PG")
	load_dim(P["ECPL"],  "ECPL")
	load_dim(P["ECPW"],  "ECPW")
	load_dim(P["PRP"],  "PRP")

	ipc_init_gentol()

	if ((P["PC"] == "") && (P["PG"] == "")) {
		print "Either PC or PG needs to be specified" > "/dev/stderr"
		exit 1
	}
	else if (P["PG"] == "") {
		MAX["PG"] = MAX["PE"] - 2*MIN["PC"]
		MIN["PG"] = MIN["PE"] - 2*MAX["PC"]
		mktol("PG")
	}
	else if (P["PC"] == "") {
		MAX["PC"] = (MAX[PE] - MIN["PG"]) / 2
		MIN["PC"] = (MIN[PE] - MAX["PG"]) / 2
		mktol("PC")
	}

# compute pad geo
	pad_geo_flat_no_lead(PADGEO, P["density"])
	so_pinpat(PINPAT, P["N"])
	Zmax = ipc_calc_extent(PADGEO, "PE")
	Gmin = ipc_calc_gap(PADGEO, "PE", "PG")
	Xmax = ipc_calc_pin_thick(PADGEO, "PT")

#print "Zmax=" Zmax, "Gmin=" Gmin > "/dev/stderr"

	# set up and call draw function
	D["pitch"] = NOM["PP"]
	D["pad_spacing"] = (Zmax+Gmin)/2
	D["pad_length"] = (Zmax-Gmin)/2
	D["pad_width"] = Xmax

	D["lead_width"] = NOM["PT"]
	D["lead_width_max"] = MAX["PT"]
	D["lead_extent"] = NOM["PE"]
	D["lead_gap"] = NOM["PG"]
	D["body_width"] = NOM["BW"]
	D["body_width_max"] = MAX["BW"]
	D["body_length"] = NOM["BL"]
	D["body_length_max"] = MAX["BL"]
	D["courtyard_extent"] = PADGEO["cy"]
	D["cpad_length"] = NOM["ECPL"]
	D["cpad_width"] = NOM["ECPW"]

	centerpad_setup(D, P)

	gullwing2_draw(D, PINPAT, "FN")
}
