
# bedrock-rnd - curated footprints, symbols and devmaps for Ringdove EDA
# Copyright (C) 2025  Tibor 'Igor2' Palinkas
#
# dist-license: GPL2+
# use-license (license of the output): unlimited (CC0 or public domain)
#
# For more information: doc/license.html
#        or on the web: http://www.repo.hu/projects/bedrock-rnd/license.html
#
# Project page: http://www.repo.hu/projects/bedrock-rnd
#

BEGIN {
	gversion="2025-01-20"
	generator = "plcc-rfp"
	bedrock_ver = "2025-01-08a"
}

BEGIN {
	help_auto()

	proc_args(P, "N,NL,NW,PW,PL,PRP,PRPW,PRPL,BS,BL,BW,PP,PE,PEL,PEW,PT,PC,PGL,PGW,CPL,CPW,CYE" ipc_params, "PL,PW")

	square_vs_rectangle_pads()

# load args
	load_dim(P["PW"],  "PW")
	load_dim(P["PL"],  "PL")
	load_dim(P["CPL"],  "CPL")
	load_dim(P["CPW"],  "CPW")
	load_dim(P["PP"],  "PP")

	load_dim(P["PT"],  "PT")
	load_dim(P["PC"],  "PC")
	load_dim(P["PGL"],  "PGL")
	load_dim(P["PGW"],  "PGW")

# improvise missing component drawing data
	if (P["CYE"] == "")
		load_rfp("CYE", mm(0.25))

	if (P["PEL"] == "")
		load_rfp("PEL", round_to(NOM["PRPL"] * 1.125, 0.05))
	if (P["PEW"] == "")
		load_rfp("PEW", round_to(NOM["PRPW"] * 1.125, 0.05))

	if ((P["PC"] == "") && (NOM["PGL"] == "")) {
		load_rfp("PGL", NOM["PRPL"] - NOM["PL"]*1.25)
		load_rfp("PGW", NOM["PRPW"] - NOM["PW"]*1.25)
		load_rfp("PC", (NOM["PEL"] - NOM["PGL"])/2.4)
	}
	if (P["PT"] == "")
		load_rfp("PT", NOM["PW"] * 0.8)
	if (P["BL"] == "")
		load_rfp("BL", round_to(NOM["PRPL"], 0.1))
	if (P["BW"] == "")
		load_rfp("BW", round_to(NOM["PRPW"], 0.1))

	if (P["PGL"] == "") {
		MAX["PGL"] = MAX["PEL"] - 2*MIN["PC"]
		MIN["PGL"] = MIN["PEL"] - 2*MAX["PC"]
		MAX["PGW"] = MAX["PEW"] - 2*MIN["PC"]
		MIN["PGW"] = MIN["PEW"] - 2*MAX["PC"]
		mktol("PGL")
		mktol("PGW")
	}
	else if (P["PC"] == "") {
		MAX["PC"] = (MAX[PEL] - MIN["PGL"]) / 2
		MIN["PC"] = (MIN[PEL] - MAX["PGL"]) / 2
		mktol("PC")
	}

# print "BODY: ", NOM["BL"], NOM["BW"] > "/dev/stderr"

	ipc_init_gentol()

	# set up and call draw function
	D["pitch"] = NOM["PP"]
	D["pad_spacing_length"] = NOM["PRPL"]
	D["pad_spacing_width"] = NOM["PRPW"]
	D["pad_length"] = NOM["PL"]
	D["pad_width"] = NOM["PW"]

	D["lead_width"] = NOM["PT"]
	D["lead_width_max"] = MAX["PT"]
	D["lead_extent_length"] = NOM["PEL"]
	D["lead_extent_width"] = NOM["PEW"]
	D["lead_gap_length"] = NOM["PGL"]
	D["lead_gap_width"] = NOM["PGL"]
	D["body_width"] = NOM["BW"]
	D["body_width_max"] = MAX["BW"]
	D["body_length"] = NOM["BL"]
	D["body_length_max"] = MAX["BL"]
	D["courtyard_extent"] = NOM["CYE"]
	D["cpad_length"] = NOM["CPL"]
	D["cpad_width"] = NOM["CPW"]

	D["pin1_top"] = 1

	centerpad_setup(D, P)

	gullwing4_draw(D, "J")
}
