
# bedrock-rnd - curated footprints, symbols and devmaps for Ringdove EDA
# Copyright (C) 2025  Tibor 'Igor2' Palinkas
#
# dist-license: GPL2+
# use-license (license of the output): unlimited (CC0 or public domain)
#
# For more information: doc/license.html
#        or on the web: http://www.repo.hu/projects/bedrock-rnd/license.html
#
# Project page: http://www.repo.hu/projects/bedrock-rnd
#


BEGIN {
	gversion="2025-01-20"
	generator = "sot23-ipc"
	bedrock_ver = "2025-01-03d"
}

BEGIN {
	help_auto()

	proc_args(P, "density,N,BL,BW,PP,PE,PT,PC,PG" ipc_params, "density,N,BL,BW,PP,PE,PT")

# load args
	load_dim(P["BL"],  "BL")
	load_dim(P["BW"],  "BW")
	load_dim(P["PP"],  "PP")
	load_dim(P["PE"],  "PE")
	load_dim(P["PT"],  "PT")
	load_dim(P["PC"],  "PC")
	load_dim(P["PG"],  "PG")
	ipc_init_gentol()

	if ((P["PC"] == "") && (P["PG"] == "")) {
		print "Either PC or PG needs to be specified" > "/dev/stderr"
		exit 1
	}
	else if (P["PG"] == "") {
		MAX["PG"] = MAX["PE"] - 2*MIN["PC"]
		MIN["PG"] = MIN["PE"] - 2*MAX["PC"]
		mktol("PG")
	}
	else if (P["PC"] == "") {
		MAX["PC"] = (MAX[PE] - MIN["PG"]) / 2
		MIN["PC"] = (MIN[PE] - MAX["PG"]) / 2
		mktol("PC")
	}

# compute pad geo
	pad_geo_gull_wing(PADGEO, P["density"], NOM["PP"])
	sot23_pinpat(PINPAT, P["N"])
	Zmax = ipc_calc_extent(PADGEO, "PE")
	Gmin = ipc_calc_gap(PADGEO, "PE", "PG")
	Xmax = ipc_calc_pin_thick(PADGEO, "PT")

#print "Zmax=" Zmax, "Gmin=" Gmin > "/dev/stderr"

	# set up and call draw function
	D["pitch"] = NOM["PP"]
	D["pad_spacing"] = (Zmax+Gmin)/2
	D["pad_length"] = (Zmax-Gmin)/2
	D["pad_width"] = Xmax

	D["lead_width"] = NOM["PT"]
	D["lead_width_max"] = MAX["PT"]
	D["lead_extent"] = NOM["PE"]
	D["lead_gap"] = NOM["PG"]
	D["body_width"] = NOM["BW"]
	D["body_width_max"] = MAX["BW"]
	D["body_length"] = NOM["BL"]
	D["body_length_max"] = MAX["BL"]
	D["courtyard_extent"] = PADGEO["cy"]

	gullwing2_draw(D, PINPAT)
}
